@@ -1,11 +1,11 @@
 /* 
  * functions.cc -- non-standard mathematical functions
  *
- * This file is part of ePiX, a C++ library for creating high-quality 
- * figures in LaTeX
+ * This file is part of ePiX, a preprocessor for creating high-quality 
+ * line figures in LaTeX 
  *
- * Version 1.0.23
- * Last Change: January 29, 2007
+ * Version 1.2.0-2
+ * Last Change: September 26, 2007
  */
 
 /* 
@@ -32,12 +32,13 @@
  * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
-#include <string>
 #include <cmath>
 
-#include "globals.h"
+#include "constants.h"
 #include "errors.h"
+#include "deriv.h"
 
+#include "angle_units.h"
 #include "triples.h"
 
 #include "functions.h"
@@ -45,36 +46,72 @@
 namespace ePiX {
 
   // trig functions with angle units
-  double cos(double t) { return std::cos(angle(t)); }
-  double sin(double t) { return std::sin(angle(t)); }
-  double tan(double t) { return std::tan(angle(t)); }
+  double Cos(double t)
+  {
+    return std::cos(the_angle_style().to_radians(t));
+  }
 
-  // non-mathematical behavior (i.e. vanishing:) at poles
-  double sec(double t) { return recip(std::cos(angle(t))); }
-  double csc(double t) { return recip(std::sin(angle(t))); }
-  double cot(double t) { return recip(std::tan(angle(t))); }
+  double Sin(double t)
+  {
+    return std::sin(the_angle_style().to_radians(t));
+  }
+
+  double Tan(double t)
+  {
+    return std::tan(the_angle_style().to_radians(t));
+  }
+
+  double Sec(double t)
+  {
+    return 1.0/std::cos(the_angle_style().to_radians(t));
+  }
+
+  double Csc(double t)
+  {
+    return 1.0/std::sin(the_angle_style().to_radians(t));
+  }
+
+  double Cot(double t)
+  {
+    return 1.0/std::tan(the_angle_style().to_radians(t));
+  }
 
   // and inverses
-  double acos(double arg) { return std::acos(arg)/angle(1); }
-  double asin(double arg) { return std::asin(arg)/angle(1); }
-  double atan(double arg) { return std::atan(arg)/angle(1); }
+  double Acos(double arg)
+  {
+    return std::acos(arg)/the_angle_style().to_radians(1);
+  }
 
-  double Cos(double t) { return ePiX::cos(t); }
-  double Sin(double t) { return ePiX::sin(t); }
-  double Tan(double t) { return ePiX::tan(t); }
+  double Asin(double arg)
+  {
+    return std::asin(arg)/the_angle_style().to_radians(1);
+  }
 
-  double Sec(double t) { return ePiX::sec(t); }
-  double Csc(double t) { return ePiX::csc(t); }
-  double Cot(double t) { return ePiX::cot(t); }
+  double Atan(double arg)
+  {
+    return std::atan(arg)/the_angle_style().to_radians(1);
+  }
 
-  double Acos(double t)  { return ePiX::acos(t); }
-  double Asin(double t)  { return ePiX::asin(t); }
-  double Atan(double t)  { return ePiX::atan(t); }
+  double Atan2(double y, double x)
+  {
+    return std::atan2(y, x)/the_angle_style().to_radians(1);
+  }
 
   // Additional hyperbolic tric functions and inverses
-  double sech(double x) { return 1.0/cosh(x); }
-  double csch(double x) { return 1.0/sinh(x); }
-  double coth(double x) { return 1.0/tanh(x); }
+  double sech(double x)
+  {
+    return 1.0/cosh(x);
+  }
+
+  double csch(double x)
+  {
+    return 1.0/sinh(x);
+  }
+
+  double coth(double x)
+  {
+    return 1.0/tanh(x);
+  }
 
   double asech(double x)
   {
@@ -91,6 +128,25 @@ namespace ePiX {
     return 0.5*log((x+1)/(x-1));
   }
 
+  double zero(double arg)
+  {
+    return 0;
+  }
+
+  double zero(double x, double y)
+  {
+    return 0;
+  }
+
+  double zero(double x, double y, double z)
+  {
+    return 0;
+  }
+
+  double zero(const P& arg)
+  {
+    return 0;
+  }
 
   P xyz(double x, double y, double z) 
   {
@@ -99,7 +155,7 @@ namespace ePiX {
 
   P cyl(double r, double t, double z) 
   {
-    return P(r*ePiX::cos(t), r*ePiX::sin(t), z);
+    return P(r*Cos(t), r*Sin(t), z);
   }
 
   P sph(double r, double t, double phi) 
@@ -107,25 +163,6 @@ namespace ePiX {
     return P(r*(Cos(t))*(Cos(phi)), r*(Sin(t))*(Cos(phi)), r*(Sin(phi)));
   }
 
-  P cylindrical(P arg)
-  {
-    return cyl(arg.x1(), arg.x2(), arg.x3());
-  }
-
-  P spherical(P arg)
-  {
-    return sph(arg.x1(), arg.x2(), arg.x3());
-  }
-
-  P polar(double r, double t)
-  {
-    return cyl(r, t, 0);
-  }
-  P cis(double t)
-  {
-    return cyl(1, t, 0);
-  }
-
   P log_log(double x, double y, double z)
   {
     return P(log10(x), log10(y), z);
@@ -141,21 +178,31 @@ namespace ePiX {
     return P(x, log10(y), z);
   }
 
-  // Force double to [0,1]
-  double clip_to_unit(double t)
+
+  // for plot templates, arg mustn't be const P&
+  P cylindrical(P arg)
   {
-    if (t < 0)       return 0;
-    else if (t > 1)  return 1;
-    else             return t;
+    return cyl(arg.x1(), arg.x2(), arg.x3());
+  }
+
+  P spherical(P arg)
+  {
+    return sph(arg.x1(), arg.x2(), arg.x3());
+  }
+
+  P polar(double r, double t)
+  {
+    return cyl(r, t, 0);
+  }
+
+  P cis(double t)
+  {
+    return cyl(1, t, 0);
   }
 
-  // reciprocal, truncated to 0 near 0
   double recip (double x)
   {
-    if (fabs(x) < 1.0/EPIX_INFTY)
-      return 0;
-    else
-      return 1/x;
+    return 1.0/x;
   }
 
   // sin(x)/x
@@ -164,7 +211,7 @@ namespace ePiX {
     if (1. + x*x == 1.) // from Don Hatch
       return 1.;
     else
-      return Sin(x)/angle(x);
+      return Sin(x)/the_angle_style().to_radians(x);
   }
 
   // signum, x/|x|, defined to be 0 at 0
@@ -215,16 +262,17 @@ namespace ePiX {
     }
   }
 
-  double min(const double a, const double b)
+  double min(double a, double b)
   {
     return a < b ? a : b;
   }
-  double max(const double a, const double b)
+
+  double max(double a, double b)
   {
     return b < a ? a : b;
   }
 
-  double snip_to(double var, const double arg1, const double arg2)  
+  double snip_to(double var, double arg1, double arg2)  
   {
     if (var < min(arg1, arg2))
       var = min(arg1,arg2);
@@ -240,7 +288,7 @@ namespace ePiX {
     double y(f(a));
     const double dx((b-a)/(N*EPIX_ITERATIONS));
 
-    for (int i=1; i <= N*EPIX_ITERATIONS; ++i)
+    for (unsigned int i=1; i <= N*EPIX_ITERATIONS; ++i)
       y = min(y, f(a + i*dx));
 
     return y;
@@ -252,7 +300,7 @@ namespace ePiX {
     double y(f(a));
     const double dx((b-a)/(N*EPIX_ITERATIONS));
 
-    for (int i=1; i <= N*EPIX_ITERATIONS; ++i)
+    for (unsigned int i=1; i <= N*EPIX_ITERATIONS; ++i)
       y = max(y, f(a + i*dx));
 
     return y;
@@ -267,7 +315,7 @@ namespace ePiX {
   Integral::Integral(double func(double), double a)
     : f(func), x0(a) { }
 
-  double Integral::eval(const double t) const
+  double Integral::eval(double t) const
   {
     double sum(0); 
     const int N(16*(int)ceil(fabs(t - x0))); // hardwired constant 16
@@ -294,7 +342,7 @@ namespace ePiX {
     double guess(start);
     int count(0); // number of iterations
 
-    // Hardwired constant 5
+    // Magic number 5
     const int ITERS(5);
     while ( (fabs(f(guess)-g(guess)) > EPIX_EPSILON) && (count < ITERS) )
       {
@@ -320,7 +368,10 @@ namespace ePiX {
   }
 
   // Member functions
-  Deriv::Deriv(double func(double), const double eps)
+  Deriv::Deriv(double func(double))
+    : f(func), dt(EPIX_EPSILON) { }
+
+  Deriv::Deriv(double func(double), double eps)
     : f(func), dt(eps) { }
 
   P Deriv::operator() (const P& arg) const
@@ -329,18 +380,18 @@ namespace ePiX {
     return P(t, deriv(f, t, dt), 0);
   }
 
-  double Deriv::eval(const double t) const
+  double Deriv::eval(double t) const
   {
     return deriv(f, t, dt);
   }
 
   // one-sided derivatives
-  double Deriv::right(const double t) const
+  double Deriv::right(double t) const
   {
     return  (2.0/dt)*(f(t+0.5*dt) - f(t));
   }
 
-  double Deriv::left(const double t) const
+  double Deriv::left(double t) const
   {
     return (2.0/dt)*(f(t) - f(t-0.5*dt));
   }
